package de.ids_mannheim.korap.openthesaurus;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class ThesaurusDictionaryTest {

    @Test
    @DisplayName("Regex applied to text fields should remove text in parentheses and strip whitespaces")
    void cleanStringTest() {
        String[][] regexStrings = new String[][] {
                {"(some annot.)(some other) Word1 Word2 Word3    (annot.)(other)", "Word1 Word2 Word3"},
                {"mit Füßen treten (ein Recht) (ugs., fig.)", "mit Füßen treten"},
                {"Atomspaltung", "Atomspaltung"},
                //{" Word ","Word"} // doesn't pass yet!
        };
        for (String[] pair : regexStrings) {
            String source = pair[0];
            String target = pair[1];
            assertEquals(target, ThesaurusDictionary.cleanString(source));
        }
    }

    <K, V extends List> boolean twoMapsAreEqual(Map<K, V> map1, Map<K, V> map2) {
        for (Map.Entry<K, V> pair : map1.entrySet()) {
            K keyTarget = pair.getKey();
            V valuesTarget = pair.getValue();
            if (!map2.containsKey(keyTarget)) return false;
            else {
                V valuesCompare = map2.get(keyTarget);
                if (!valuesCompare.containsAll(valuesTarget)) return false;
            }
        }
        return true;
    }

    @Test
    @DisplayName("Test HashMaps created from lines")
    void processLineTest() {
        String source = "unhinterfragt;unkritisch;bedenkenlos";
        Map<String, List<String>> target = new HashMap<>();
        target.put("unhinterfragt", List.of(new String[]{
                "unkritisch", "bedenkenlos"
        }));
        target.put("unkritisch", List.of(new String[]{
                "unhinterfragt", "bedenkenlos"
        }));
        target.put("bedenkenlos", List.of(new String[]{
                "unhinterfragt", "unkritisch"
        }));

        String source1 = "abhängen (von);(sich) richten (nach);(sich) bemessen (nach)";
        Map<String, List<String>> target1 = new HashMap<>();
        target1.put("abhängen", List.of(new String[]{
                "richten", "bemessen"
        }));
        target1.put("richten", List.of(new String[]{
                "abhängen", "bemessen"
        }));
        target1.put("bemessen", List.of(new String[]{
                "abhängen", "richten"
        }));

        assertTrue(twoMapsAreEqual(target, ThesaurusDictionary.processLine(source)));
        assertTrue(twoMapsAreEqual(target1, ThesaurusDictionary.processLine(source1)));

    }
}
